#Author: Edward Oh using Doug's code posted online
#Filename: GameEngineFINAL.py


import time, random
import textwrap

class State:
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.edges = []
        self.objects = []

class Edge:
    def __init__(self, name, from_node, commands, to_node, description, healthchange, constraint=None):
        self.name = name
        self.description = description
        self.from_node = from_node 
        self.commands = commands
        self.to_node = to_node
        self.constraint = constraint
        self.accessconstraint = []
        self.healthchange = int(healthchange)

class Constraint:
    def __init__(self, name, object_name, state_name):
        self.name = name
        self.object_name = object_name
        self.state_name = state_name

class Object:
    def __init__(self, location, name, shortdescription, description, weight=None):
        self.location = location
        self.name = name
        self.shortdescription = shortdescription
        self.description = description
        self.weight = weight

class BackPack:
    def __init__(self, maxweight, value):
        self.maxweight = maxweight
        self.value = value


class Engine:
    def __init__(self, filename):
        self.filename = filename
        self.initial_state = None
        self.counter = 0
        self.health = 10
        self.time = time.time()
        self.backpack = None
    
    def load(self):
        self.states = {}
        self.backpackitems = {}
        self.edges = {}
        self.objects = {}
        edges = []
        objects = []
        backpack= []
        constraints = []
        fp = open(self.filename)
        for line in fp:
            if len(line) == 0 or line[0] == '#':
                continue
            items = line.strip().split("::")
            if items[0] == 'node':
                s = State(*items[1:])
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
            elif items[0] == 'edge':
                edges.append(Edge(*items[1:])) #what does *items do? -Takes each item and puts it in edge
            elif items[0] == 'object': #load objects here
                objects.append(Object(*items[1:]))
            elif items[0] == 'backpack':
                self.backpack = BackPack(*items[1:])
            elif items[0] == 'constraint':
                constraints.append(Constraint(*items[1:]))
            
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
            for constraint in constraints:
                if constraint.name == edge.constraint:
                    index = self.states[edge.from_node].edges.index(edge)
                    self.states[edge.from_node].edges[index].accessconstraint.append(constraint)
                 
        for obj in objects:
            self.states[obj.location].objects.append(obj) #appends object of that specific location to the state of that location #does this work??
            self.objects[obj.name] = obj

            
    def play(self):
        current_state = self.initial_state
        CommandRecognized = 1
        TempCommandRecognized = 1
        OfferHelpCounter = 0
        Name = raw_input("Please enter your name: ")
        
        while True:
            if CommandRecognized == 1:
                TempCommandRecognized = 0
                print TextWrapping(current_state.description)
                for obj in current_state.objects: #checks if object is in room, if so, prints short description
                    print obj.shortdescription
                print "Counter: ", self.counter
                print "Health: ", self.health
                print "Time: ", time.time() - self.time, "seconds"
            CommandRecognized = 0 #Variable use to determine whether "Command not recognized..." should be printed or not
            command = raw_input("Command> ")
            
            #EDGES
            for edge in current_state.edges:
                if command in edge.commands:
                    if edge.constraint == "None":
                        CommandRecognized = 1
                        OfferHelpCounter = 0
                        print TextWrapping(edge.description)
                        current_state = self.states[edge.to_node]
                        self.counter = self.counter + 1
                        self.health = self.health + edge.healthchange
                        break
                    else:
                        #Constraints
                        for constraint in edge.accessconstraint:
                            #Eval
                            if constraint.object_name == "eval":
                                GoThrough = 0
                                if eval(constraint.state_name)==True:
                                    CommandRecognized = 1
                                    print TextWrapping(edge.description)
                                    current_state = self.states[edge.to_node]
                                    self.counter = self.counter + 1
                                    self.health = self.health + edge.healthchange
                                    GoThrough = 1
                                else:
                                    print "You have to do sth before going through here!"
                            #Exec
                            elif constraint.object_name == "exec":
                                GoThrough = 1
                                exec constraint.state_name
                                CommandRecognized = 1
                                print TextWrapping(edge.description)
                                current_state = self.states[edge.to_node]
                                self.counter = self.counter + 1
                                self.health = self.health + edge.healthchange
                                GoThrough = 1
                            #Objects
                            else:
                                for obj in self.objects:
                                    GoThrough = 0
                                    if self.objects[obj].location == constraint.state_name:
                                        CommandRecognized = 1
                                        print TextWrapping(edge.description)
                                        current_state = self.states[edge.to_node]
                                        self.counter = self.counter + 1
                                        self.health = self.health + edge.healthchange
                                        GoThrough = 1
                                        break
                            if GoThrough == 0:
                                print Name, "you cannot go through!"
                
            #PICK UP OBJECT
            for obj in current_state.objects:
                if command == "pick up "+ obj.name or command == "pickup " + obj.name:
                    TempCommandRecognized = 1
                    if int(self.backpack.value) + int(obj.weight) > int(self.backpack.maxweight):
                        print "Cannot pick up item, pack is too heavy and ", Name, " is too weak!"
                    else:
                        self.backpackitems[obj.name] = obj
                        self.states[obj.location].objects.remove(obj)
                        print "Object picked up"
                        obj.location = "backpack"
                    
            #DROP OBJECT
            for obj in self.backpackitems:
                if command == "drop "+ self.backpackitems[obj].name:
                    TempCommandRecognized = 1
                    self.backpackitems[obj].location = current_state.name
                    self.states[self.backpackitems[obj].location].objects.append(self.backpackitems[obj])
                    del self.backpackitems[obj]
                    break
                    
            #LOOK OBJECT
            for obj in current_state.objects:
                if command == "look "+ obj.name:
                    TempCommandRecognized = 1
                    print TextWrapping(obj.description)
                    
            #CHECK BACKPACK
            if command == "backpack":
                TempCommandRecognized = 1
                print "The following items are in your inventory: "
                print self.backpackitems.keys()
                
            #HEALTH
            if self.health <= 0:
                print Name, ", you died"
                break
            
            #HINT
            if command == "hint" or command == "Hint":
                print "Here are the objects in the room: "
                for obj in current_state.objects:
                    print obj.name
                TempCommandRecognized = 1

            #QUIT
            if command == "quit" or command == "Quit":
                print "Good bye", Name, "!"
                break
            
            #COMMAND NOT RECOGNIZED AND OFFER HELP            
            if CommandRecognized == 0 and TempCommandRecognized == 0:    
                print "You can't do that!"
                OfferHelpCounter = OfferHelpCounter + 1
                if OfferHelpCounter >= 3:
                    Help = raw_input("Do you need help?[y/n]: ")
                    if Help == "y":
                        print "Maybe this will help..."
                        for edge in current_state.edges:
                            print "Available commands: ", edge.commands
                        OfferHelpCounter = 0
                    else:
                        print "Of course you don't need help!"
                        OfferHelpCounter = 0
                    
def TextWrapping(text):
        return textwrap.fill(text)

game = Engine("InteractiveFictionStory2.txt")
game.load()
game.play()

    
